package com.monkeycoder.monkeynes.gui;

import java.util.*;
import java.awt.event.*;

/**
 * This class acts as a mapping between keyboard keys and NES controller buttons.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class ButtonMap
{
  private HashMap map = new HashMap( 8 );
  
  /**
   * Creates a new empty ButtonMap.
   */
  public ButtonMap()
  {
    // Do nothing...
  }
  
  /**
   * Adds a new mapping into this ButtonMap.
   * @param inKeyCode The keyboard key code.
   * @param inButton The ID of the NES button.
   */  
  public void addMapping( int inKeyCode, int inButton )
  {
    map.put( new Integer( inKeyCode ), new Integer( inButton ) );
  }

  /**
   * Gets the NES button ID for a given key code.  Returns -1 if there is no 
   * NES button that maps to the given key code.   
   * @param inKeyCode The keyboard key code.
   * @return The NES button ID or -1 to indicate there was no mapping.
   */   
  public int getButton( int inKeyCode )
  {
    Integer mapValue = (Integer)map.get( new Integer( inKeyCode ) );
    if( mapValue != null )
      return mapValue.intValue();
    else
      return -1;
  }

  /**
   * Creates a String containing all the mappings in this ButtonMap.
   * @return The String of the mappings.
   */   
  public String toString()
  {
    StringBuffer sb = new StringBuffer();

    Object[] keys = map.keySet().toArray();
    sb.append( "ButtonMap\n" );
    for( int i = 0; i < keys.length; i++ )
    {
      sb.append( "  KeyCode: " );
      sb.append( ((Integer)keys[i]).intValue() );
      sb.append( " -> Button: " );
      sb.append( ((Integer)map.get( keys[i] )).intValue() );
      sb.append( "\n" );
    }
    
    return sb.toString();
  }
}