package com.monkeycoder.monkeynes.gui;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.hardware.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * This is a GUI Dialog box that displays the current state of all the 
 * CPU registers.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class CpuStateViewerGui
{
  private Frame              parentFrame                      = null;
  private JDialog            mainDialog                       = null;
  private NesCpu             cpu                              = null;
  
  private JPanel             registerPanel                    = new JPanel( new GridLayout( 6, 2 ) );
  private JLabel             pcLabel                          = new JLabel( "$0" );
  private JLabel             spLabel                          = new JLabel( "$0" );
  private JLabel             accLabel                         = new JLabel( "$0" );
  private JLabel             ixLabel                          = new JLabel( "$0" );
  private JLabel             iyLabel                          = new JLabel( "$0" );
  
  private JPanel             flagPanel                        = new JPanel( new GridLayout( 2, 8 ) );
  private JLabel             flagSLabel                       = new JLabel( "0" );
  private JLabel             flagVLabel                       = new JLabel( "0" );
  private JLabel             flagXLabel                       = new JLabel( "1" );
  private JLabel             flagBLabel                       = new JLabel( "0" );
  private JLabel             flagDLabel                       = new JLabel( "0" );
  private JLabel             flagILabel                       = new JLabel( "0" );
  private JLabel             flagZLabel                       = new JLabel( "0" );
  private JLabel             flagCLabel                       = new JLabel( "0" );

  private JButton            updateButton                     = new JButton( "Update" );

  /** 
   * Creates and configures a new CpuStateViewerGui.
   * Note that the window is constructed but not displayed.
   * @param inParentFrame The parent of this dialog.
   * @param inCpu The CPU for which this dialog will display values.
   */     
  public CpuStateViewerGui( Frame inParentFrame, NesCpu inCpu )
  {
    parentFrame = inParentFrame;
    cpu         = inCpu;
    
    // Dialog
    mainDialog = new JDialog( parentFrame, "CPU State", false );
    
    // Put together the flag panel
    flagPanel.add( new JLabel( "S" ) );
    flagPanel.add( new JLabel( "V" ) );
    flagPanel.add( new JLabel( "-" ) );
    flagPanel.add( new JLabel( "B" ) );
    flagPanel.add( new JLabel( "D" ) );
    flagPanel.add( new JLabel( "I" ) );
    flagPanel.add( new JLabel( "Z" ) );
    flagPanel.add( new JLabel( "C" ) );
    flagPanel.add( flagSLabel );
    flagPanel.add( flagVLabel );
    flagPanel.add( flagXLabel );
    flagPanel.add( flagBLabel );
    flagPanel.add( flagDLabel );
    flagPanel.add( flagILabel );
    flagPanel.add( flagZLabel );
    flagPanel.add( flagCLabel );
    
    // Put together the register panel
    registerPanel.add( new JLabel( "PC:" ) );
    registerPanel.add( pcLabel );
    registerPanel.add( new JLabel( "SP:" ) );
    registerPanel.add( spLabel );
    registerPanel.add( new JLabel( "ACC:" ) );
    registerPanel.add( accLabel );
    registerPanel.add( new JLabel( "IX:" ) );
    registerPanel.add( ixLabel );
    registerPanel.add( new JLabel( "IY:" ) );
    registerPanel.add( iyLabel );
    registerPanel.add( new JLabel( "PS:" ) );
    registerPanel.add( flagPanel );
    
    // Build the main view
    mainDialog.getContentPane().add( registerPanel,     BorderLayout.CENTER  );
    mainDialog.getContentPane().add( updateButton,      BorderLayout.SOUTH  );

    // Update button listener
    updateButton.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        pcLabel.setText(  "$" + Integer.toString( cpu.getProgramCounterValue(), 16 ).toUpperCase() );
        spLabel.setText(  "$" + Integer.toString( cpu.getStackPointerValue(), 16 ).toUpperCase() );
        accLabel.setText( "$" + Integer.toString( cpu.getAccumulatorValue(), 16 ).toUpperCase() );
        ixLabel.setText(  "$" + Integer.toString( cpu.getXIndexValue(), 16 ).toUpperCase() );
        iyLabel.setText(  "$" + Integer.toString( cpu.getYIndexValue(), 16 ).toUpperCase() );
        
        CpuRegister ps = new CpuRegister();
        ps.setValue( cpu.getProcessorStatusValue() );
        flagSLabel.setText( String.valueOf( ps.getBitValue( CpuRegister.PS_NEGATIVE_FLAG     ) ) );
        flagVLabel.setText( String.valueOf( ps.getBitValue( CpuRegister.PS_OVERFLOW_FLAG     ) ) );
        flagXLabel.setText( String.valueOf( ps.getBitValue( CpuRegister.PS_UNUSED            ) ) );
        flagBLabel.setText( String.valueOf( ps.getBitValue( CpuRegister.PS_BREAK_COMMAND     ) ) );
        flagDLabel.setText( String.valueOf( ps.getBitValue( CpuRegister.PS_DECIMAL_MODE      ) ) );
        flagILabel.setText( String.valueOf( ps.getBitValue( CpuRegister.PS_INTERRUPT_DISABLE ) ) );
        flagZLabel.setText( String.valueOf( ps.getBitValue( CpuRegister.PS_ZERO_FLAG         ) ) );
        flagCLabel.setText( String.valueOf( ps.getBitValue( CpuRegister.PS_CARRY_FLAG        ) ) );
      }
    } );

    // Main window closing listener
    mainDialog.addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent we )
      {
        mainDialog.hide();
      }
    } );
  } // End of constructor ----------------------------------------------------------

  /**
   * Displays the CpuStateViewerGui.
   */  
  public void show()
  {
    // Position the window
    Dimension parentSize = parentFrame.getSize();
    Point     parentLoc  = parentFrame.getLocation();
    Dimension mySize     = new Dimension( 250, 200 );
    Point     myLoc      = new Point( parentLoc.x, parentLoc.y );
    
    // Fix the location in case the positioning put part of the window off the screen                                      
    if( myLoc.x < 0 ) myLoc.x = 0;                                      
    if( myLoc.y < 0 ) myLoc.y = 0;
    
    // Size and position the window
    mainDialog.setLocation( myLoc );
    mainDialog.setSize( mySize );
 
    // Show the whole mess...
    mainDialog.show();
  }
}