package com.monkeycoder.monkeynes.gui;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.hardware.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * This is the GUI dialog that a hex view of the contents of the PRG-ROM.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class PrgRomHexViewerGui implements Observer
{
  private Frame              parentFrame                      = null;
  private JDialog            mainDialog                       = null;
    
  private NesMainMemory      mainMem                          = null;    
  private boolean            visible                          = false;
  
  private Document    codeDoc       = new DefaultStyledDocument();
  private JTextArea   codeArea      = new JTextArea( codeDoc, "", 15, 30 );
  private JScrollPane codeScroll    = new JScrollPane( codeArea );

  /** 
   * Creates and configures a new PatternTableViewerGui.
   * Note that the window is constructed but not displayed.
   * @param inParentFrame The parent of this dialog.
   * @param inMainMemory The MainMemory from which the PRG-ROM will be loaded.   
   */  
  public PrgRomHexViewerGui( Frame inParentFrame, NesMainMemory inMainMemory )
  {
    parentFrame = inParentFrame;
    mainMem     = inMainMemory;
    
    mainMem.addObserver( this );
    
    codeArea.setFont( new Font( "Courier", Font.PLAIN, 12 ) );
    codeArea.setEditable( false );
    
    // Dialog
    mainDialog = new JDialog( parentFrame, "PRG-ROM Hex View", false );
    
    // Build the main view
    mainDialog.getContentPane().add( codeScroll,     BorderLayout.CENTER  );

    // Main window closing listener
    mainDialog.addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent we )
      {
        mainDialog.hide();
        visible = false;
      }
    } );
  } // End of constructor ----------------------------------------------------------
  
  /**
   * Accepts messages from main memory.  If the specific area of main memory that 
   * contains the PRG-ROM changes a redraw of this control will be triggered.
   * @param o The observable object sending the message.
   * @param arg The message being sent.
   */  
  public void update( Observable o, Object arg )
  {
    MemoryAccessMessage mess = (MemoryAccessMessage)arg;
    
    DebugLog.log( 1, mess.toString() );
    
    if( visible && 
        mess.getMemoryBank() == MemoryAccessMessage.MEM_MAIN && 
        ( mess.getType() == MemoryAccessMessage.TYPE_CLEAR ||
          ( mess.getType() == MemoryAccessMessage.TYPE_WRITE || 
            mess.getType() == MemoryAccessMessage.TYPE_WRITE_WORD || 
            mess.getType() == MemoryAccessMessage.TYPE_WRITE_RANGE ) &&
              ( mess.getAddrStart() >= 0x00008000 || 
                mess.getAddrStart() <= 0x0000FFFF 
              ) 
          )
      )  
    {
      createHexView();
    }
  }
  
  private void createHexView()
  {
    StringBuffer sb = new StringBuffer();
    int opCodeNum = 0;
    
    for( int i = 0x00008000; i <= 0x0000FFFF; i+=16 )
    {
      // Output the address
      sb.append( "$" );
      sb.append( MathHelper.getHexString16( i ) );
      sb.append( ": " );
      
      for( int j = 0; j < 16; j++ )
      {
        sb.append( MathHelper.getHexString8( mainMem.readNoMessage( i + j ) ) );
        sb.append( " " );
      }  
        sb.append( "\n" );
    }
    
    codeArea.setText( "" );
    codeArea.append( sb.toString() );
    
    codeArea.setCaretPosition( 0 );
  }

  /**
   * Displays the PrgRomHexViewerGui.
   */  
  public void show()
  {
    // Position the window
    Dimension parentSize = parentFrame.getSize();
    Point     parentLoc  = parentFrame.getLocation();
    Dimension mySize     = new Dimension( 430, 600 );
    Point     myLoc      = new Point( parentLoc.x, parentLoc.y );
    
    // Fix the location in case the positioning put part of the window off the screen                                      
    if( myLoc.x < 0 ) myLoc.x = 0;                                      
    if( myLoc.y < 0 ) myLoc.y = 0;
    
    // Size and position the window
    mainDialog.setLocation( myLoc );
    mainDialog.setSize( mySize );
 
    createHexView();
    
    // Show the whole mess...
    mainDialog.show();
    
    visible = true;
  }

}