package com.monkeycoder.monkeynes.hardware;

import java.util.*;

/**
 * This class fulfills the MemoryFilter interface and does no translation
 * of values.  Please note that MemoryFilters apply ONLY to single byte 
 * reads and writes or byte range reads and writes.  No filtering is done
 * for "word" reads or writes.
 * <br><br>
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class DefaultMemoryFilter implements MemoryFilter
{
  /**
   * Creates a new DefaultMemoryFilter.
   */
  public DefaultMemoryFilter()
  {
    // Do nothing...
  }

  /** 
   * This function simply returns the value in memory at the requested address.
   * @param inAddress The address from which the value was read.
   * @param inValue A value already read from memory.
   * @return The filtered value at that address.
   */
  public int getFilteredReadValue( int inAddress, int inValue )
  {
    return inValue;
  }

  /** 
   * This function simply returns back the value passed in as the value to write.
   * @param inAddress The address of the requested value.
   * @param inValue The value that was going to be written.
   * @return The value to write at the indicated address.
   */
  public int getFilteredWriteValue( int inAddress, int inValue )
  {
    return inValue;
  }
}