package com.monkeycoder.monkeynes.hardware;

/**
 * This class represents an instruction that was just executed by the CPU.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class ExecutedInstructionInfo
{
  private int cycles = 0;    // The actual number of cycles, as executed.
  private int opcode = 0;    // The OpCode executed
  private int param1 = 0;    // byte 1 of the param 
  private int param2 = 0;    // byte 2 of the param
  
  /**
   * Creates a new ExecutedInstructionInfo record with the supplied information.
   * @param inCycles The number of cycles this instruction actually took in the CPU.
   * @param inOpCode The OpCode that was executed.
   * @param inParam1 The first byte of parameter data used by the instruction.
   * @param inParam2 The second byte of parameter data used by the instruction.
   */
  public ExecutedInstructionInfo( int inCycles, int inOpCode, int inParam1, int inParam2 )
  {
    cycles = inCycles;
    opcode = inOpCode;
    param1 = inParam1;
    param2 = inParam2;
  }
  
  /**
   * Gets the number of cycles the CPU took to execute this instruction.
   * @return The number of cycles the CPU took to execute this instruction.
   */
  public int getCycles()
  {
    return cycles;
  }

  /**
   * Gets the index of the OpCode that was executed.
   * @return The index of the OpCode that was executed.
   */  
  public int getOpCode()
  {
    return opcode;
  }

  /**
   * Gets the first byte of param data used by this executed instruction.
   * @return The first byte of param data used by this executed instruction.
   */  
  public int getParam1()
  {
    return param1;
  }

  /**
   * Gets the second byte of param data used by this executed instruction.
   * @return The second byte of param data used by this executed instruction.
   */  
  public int getParam2()
  {
    return param2;
  }

  /**
   * Produces a String containing the 6502 assembly language version of this instruction
   * as it was executed.
   * @return The 6502 assembly formatted code.
   */   
  public String getSyntaxFormatted()
  {
    return NesOpCodeTable.getSyntaxFormatted( opcode, param1, param2 );
  }
}