package com.monkeycoder.monkeynes.hardware;

import java.util.*;

/**
 * This is a hardware component representing the timing circuit in the NES.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class HighResolutionTimer
{
  // All clock speeds are in Hz
  private long    clockSpeed  = 0L;
  private double  clockFactor = 0.0;
  private long    lastClock   = 0L;
  private boolean running     = false;

  /**
   * Creates a new HighResolutionTimer at the specified speed.
   * @param inClockSpeed The speed of this timer in Hz.
   */
  public HighResolutionTimer( long inClockSpeed )
  {
    setClockSpeed( inClockSpeed );  
  }

  /**
   * Sets the speed of this timer to the specified speed.
   * @param inClockSpeed The speed of this timer in Hz.
   */
  public void setClockSpeed( long inClockSpeed )
  {
    clockSpeed  = inClockSpeed;
    clockFactor = (clockSpeed / 1000.0);
  }

  /**
   * Gets the speed of this timer.
   * @return The speed of this timer in Hz.
   */
  public long getClockSpeed()
  {
    return clockSpeed;
  }

  /**
   * Starts this timer.  This is required to activate and initialize the timer.
   */  
  public void startClock()
  {
    lastClock = System.currentTimeMillis();
    running   = true;
  }

  /**
   * Stops this timer.
   */  
  public void stopClock()
  {
    running   = false;
  }  
  
  /**
   * Returns whether or not this timer is running.
   * @return Whether or not the timer is running.
   */  
  public boolean isRunning()
  {
    return running;
  }

  /**
   * Gets how many cycles have elapsed since the last this this function was called.  This
   * number is controlled by the speed to which this timer is set.
   * @return The number of cycles that have elapsed.
   */  
  public long getClocksPast()
  {
    long nowClock = System.currentTimeMillis();
    long retClocks = (int)((nowClock - lastClock) * clockFactor);
    
    lastClock = nowClock;
    
    return retClocks;
  }
}