package com.monkeycoder.monkeynes.hardware;

import com.monkeycoder.monkeynes.gui.*;

/**
 * This is the Original NES Joypad, a small square Joypad 
 * with 4-directional D-Pad, B and A buttons, and Start and 
 * Select buttons.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class NesJoypad implements NesController
{
  public static final int BUTTON_A          = 0;
  public static final int BUTTON_B          = 1;
  public static final int BUTTON_SELECT     = 2;
  public static final int BUTTON_START      = 3;
  public static final int BUTTON_DPAD_UP    = 4;
  public static final int BUTTON_DPAD_DOWN  = 5;
  public static final int BUTTON_DPAD_LEFT  = 6;
  public static final int BUTTON_DPAD_RIGHT = 7;
  public static final int BUTTON_COUNT      = 8;
  
  private boolean   strobeReset      = false;
  private boolean   strobeClear      = false;
  private boolean   sending          = false;
  private int       curButton        = -1;  // incremented before send
  private int[]     buttonValue      = new int[BUTTON_COUNT];  
  private int[]     savedButtonValue = new int[BUTTON_COUNT];  
  private ButtonMap buttonMap        = null;

  /** 
   * Creates a new NesJoypad.
   */
  public NesJoypad()
  {
    // Do nothing...
  }
  
  public int getType()
  {
    return NesController.TYPE_JOYPAD;
  }

  public String getTypeName()
  {
    return "Original NES Joypad";
  }
  
  public void setButtonValue( int inButton, int inValue )
  {
    if( inButton >= 0 && inButton < BUTTON_COUNT )
      buttonValue[inButton] = inValue;
  }  
  
  public void resetStrobe()
  {
    strobeReset = true;
    strobeClear = false;
    sending     = false;
  }
  
  public void clearStrobe()
  {
    strobeClear = true;
    
    if( strobeReset && strobeClear )
    {
      strobeReset = false;
      strobeClear = false;
      sending     = true;
      curButton   = -1;
      
      // Move the current button values to a copy to prevent currupted reads from new button presses
      for( int i = 0; i < buttonValue.length; i++ )
        savedButtonValue[i] = buttonValue[i];
    }
  }
  
  public int getNextButtonValue()
  {
    if( sending )
    {
      curButton++;
      if( curButton < BUTTON_COUNT )
      {
        return savedButtonValue[curButton];
      }
      else
      {
        sending = false;
      }  
    }
    return 0;
  }
}